@@ -1,8 +1,5 @@
-/* $Id: scan.c,v 1.15 2002/08/02 19:26:55 adam Exp $
-   Copyright (C) 1995,1996,1997,1998,1999,2000,2001,2002
-   Index Data Aps
-
-This file is part of the Zebra server.
+/* This file is part of the Zebra server.
+   Copyright (C) 1995-2008 Index Data
 
 Zebra is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
@@ -15,9 +12,9 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with Zebra; see the file LICENSE.zebra.  If not, write to the
-Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
-02111-1307, USA.
+along with this program; if not, write to the Free Software
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+
 */
 
 
@@ -26,12 +23,12 @@ Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 #include <stdio.h>
 #include <assert.h>
 
-#include <dict.h>
+#include "dict-p.h"
 
-int dict_scan_trav (Dict dict, Dict_ptr ptr, int pos, Dict_char *str, 
-		    int start, int *count, void *client,
-                    int (*userfunc)(char *, const char *, int, void *),
-		    int dir)
+static void scan_direction(Dict dict, Dict_ptr ptr, int pos, Dict_char *str, 
+                           int start, int *count, void *client,
+                           int (*userfunc)(char *, const char *, int, void *),
+                           int dir)
 {
     int lo, hi, j;
     void *p;
@@ -63,9 +60,13 @@ int dict_scan_trav (Dict dict, Dict_ptr ptr, int pos, Dict_char *str,
             for (j = 0; info[j] != DICT_EOS; j++)
 		str[pos+j] = info[j];
             str[pos+j] = DICT_EOS;
-            (*userfunc)((char*) str, info+(j+1)*sizeof(Dict_char),
-                            *count * dir, client);
-            --(*count);
+            if ((*userfunc)((char*) str, info+(j+1)*sizeof(Dict_char),
+                            *count * dir, client))
+            {
+                *count = 0;
+            }
+            else
+                --(*count);
         }
         else
         {
@@ -81,29 +82,43 @@ int dict_scan_trav (Dict dict, Dict_ptr ptr, int pos, Dict_char *str,
             memcpy (&dc, info+sizeof(Dict_ptr), sizeof(Dict_char));
             str[pos] = dc;
 	    memcpy (&subptr, info, sizeof(Dict_ptr));
-	    if (info[sizeof(Dict_ptr)+sizeof(Dict_char)])
+	    if (dir>0 && info[sizeof(Dict_ptr)+sizeof(Dict_char)])
             {
                  str[pos+1] = DICT_EOS;
                  if ((*userfunc)((char*) str,
                                  info+sizeof(Dict_ptr)+sizeof(Dict_char),
                                  *count * dir, client))
-                     return 1;
-                 --(*count);
+                 {
+                     *count = 0;
+                 }
+                 else
+                     --(*count);
             }
-            if (*count > 0 && subptr)
+            if (*count>0 && subptr)
             {
-	        dict_scan_trav (dict, subptr, pos+1, str, -1, count, 
+	        scan_direction (dict, subptr, pos+1, str, -1, count, 
                                 client, userfunc, dir);
                 dict_bf_readp (dict->dbf, ptr, &p);
                 indxp = (short*) ((char*) p+DICT_bsize(p)-sizeof(short)); 
 	    }
+	    if (*count>0 && dir<0 && info[sizeof(Dict_ptr)+sizeof(Dict_char)])
+            {
+                 str[pos+1] = DICT_EOS;
+                 if ((*userfunc)((char*) str,
+                                 info+sizeof(Dict_ptr)+sizeof(Dict_char),
+                                 *count * dir, client))
+                 {
+                     *count = 0;
+                 }
+                 else
+                     --(*count);
+            }
         }
         lo += dir;
     }
-    return 0;
 }
 
-int dict_scan_r (Dict dict, Dict_ptr ptr, int pos, Dict_char *str, 
+void dict_scan_r(Dict dict, Dict_ptr ptr, int pos, Dict_char *str, 
 		 int *before, int *after, void *client,
                  int (*userfunc)(char *, const char *, int, void *))
 {
@@ -114,7 +129,7 @@ int dict_scan_r (Dict dict, Dict_ptr ptr, int pos, Dict_char *str,
 
     dict_bf_readp (dict->dbf, ptr, &p);
     if (!p)
-        return 0;
+        return;
     mid = lo = 0;
     hi = DICT_nodir(p)-1;
     indxp = (short*) ((char*) p+DICT_bsize(p)-sizeof(short));
@@ -132,11 +147,15 @@ int dict_scan_r (Dict dict, Dict_ptr ptr, int pos, Dict_char *str,
             {
                 if (*after)
                 {
-                    (*userfunc)((char *) str, info+
-                                (dict_strlen((Dict_char*) info)+1)
-                                *sizeof(Dict_char), 
-                                *after, client);
-                    --(*after);
+                    if ((*userfunc)((char *) str, info+
+                                    (dict_strlen((Dict_char*) info)+1)
+                                    *sizeof(Dict_char), 
+                                    *after, client))
+                    {
+                        *after = 0;
+                    }
+                    else
+                        --(*after);
                 }
                 break;
             }
@@ -162,23 +181,25 @@ int dict_scan_r (Dict dict, Dict_ptr ptr, int pos, Dict_char *str,
                     {
                         if (*after)
                         {
-                            (*userfunc)((char*) str,
-                                        info+sizeof(Dict_ptr)+
-                                        sizeof(Dict_char),
-                                        *after, client);
-                            --(*after);
+                            if ((*userfunc)((char*) str,
+                                            info+sizeof(Dict_ptr)+
+                                            sizeof(Dict_char),
+                                            *after, client))
+                            {
+                                *after = 0;
+                            }
+                            else
+                                --(*after);
                         }
                     }
                     if (*after && subptr)
-		        if (dict_scan_trav (dict, subptr, pos+1, str, -1, 
-                                            after, client, userfunc, 1))
-                            return 1;
+		        scan_direction(dict, subptr, pos+1, str, -1, 
+                                       after, client, userfunc, 1);
                 }
 		else if (subptr)
                 {
-                    if (dict_scan_r (dict, subptr, pos+1, str, before, after,
-                                     client, userfunc))
-                        return 1;
+                    dict_scan_r(dict, subptr, pos+1, str, before, after,
+                                client, userfunc);
                 }
                 break;
             }
@@ -191,29 +212,35 @@ int dict_scan_r (Dict dict, Dict_ptr ptr, int pos, Dict_char *str,
     if (lo>hi && cmp < 0)
         ++mid;
     if (*after)
-        if (dict_scan_trav (dict, ptr, pos, str, cmp ? mid : mid+1, after,
-                            client, userfunc, 1))
-            return 1;
+        scan_direction(dict, ptr, pos, str, cmp ? mid : mid+1, after,
+                       client, userfunc, 1);
     if (*before && mid > 0)
-        if (dict_scan_trav (dict, ptr, pos, str, mid-1, before, 
-                            client, userfunc, -1))
-            return 1;
-    return 0;
+        scan_direction(dict, ptr, pos, str, mid-1, before, 
+                       client, userfunc, -1);
 }
 
-int dict_scan (Dict dict, char *str, int *before, int *after, void *client,
-               int (*f)(char *name, const char *info, int pos, void *client))
+int dict_scan(Dict dict, char *str, int *before, int *after, void *client,
+              int (*f)(char *name, const char *info, int pos, void *client))
 {
     int i;
 
-    logf (LOG_DEBUG, "dict_scan");
+    yaz_log(YLOG_DEBUG, "dict_scan");
     for (i = 0; str[i]; i++)
     {
-	logf (LOG_DEBUG, " %3d  %c", str[i],
-	      (str[i] > ' ' && str[i] < 127) ? str[i] : '?');
+	yaz_log(YLOG_DEBUG, "start_term pos %d %3d  %c", i, str[i],
+                (str[i] > ' ' && str[i] < 127) ? str[i] : '?');
     }
     if (!dict->head.root)
         return 0;
-    return dict_scan_r (dict, dict->head.root, 0, (Dict_char *) str,
-			before, after, client, f);
+    dict_scan_r(dict, dict->head.root, 0, (Dict_char *) str,
+                before, after, client, f);
+    return 0;
 }
+/*
+ * Local variables:
+ * c-basic-offset: 4
+ * indent-tabs-mode: nil
+ * End:
+ * vim: shiftwidth=4 tabstop=8 expandtab
+ */
+
