@@ -1,8 +1,5 @@
-/* $Id: rank1.c,v 1.14 2003/03/26 16:57:24 adam Exp $
-   Copyright (C) 1995,1996,1997,1998,1999,2000,2001,2002,2003
-   Index Data Aps
-
-This file is part of the Zebra server.
+/* This file is part of the Zebra server.
+   Copyright (C) 1995-2008 Index Data
 
 Zebra is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
@@ -15,24 +12,25 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with Zebra; see the file LICENSE.zebra.  If not, write to the
-Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
-02111-1307, USA.
+along with this program; if not, write to the Free Software
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+
 */
 
-
-
 #include <stdio.h>
 #include <assert.h>
 #ifdef WIN32
 #include <io.h>
-#else
+#endif
+#if HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 
-#define DEBUG_RANK 0
-
 #include "index.h"
+#include "rank.h"
+
+static int log_level = 0;
+static int log_initialized = 0;
 
 struct rank_class_info {
     int dummy;
@@ -40,10 +38,12 @@ struct rank_class_info {
 
 struct rank_term_info {
     int local_occur;
-    int global_occur;
+    zint global_occur;
     int global_inv;
     int rank_flag;
     int rank_weight;
+    TERMID term;
+    int term_index;
 };
 
 struct rank_set_info {
@@ -51,11 +51,14 @@ struct rank_set_info {
     int no_entries;
     int no_rank_entries;
     struct rank_term_info *entries;
+    NMEM nmem;
 };
 
-static int log2_int (unsigned g)
+static int log2_int (zint g)
 {
     int n = 0;
+    if (g < 0)
+	return 0;
     while ((g = g>>1))
 	n++;
     return n;
@@ -67,10 +70,15 @@ static int log2_int (unsigned g)
  */
 static void *create (ZebraHandle zh)
 {
-    struct rank_class_info *ci = (struct rank_class_info *)
-	xmalloc (sizeof(*ci));
+    struct rank_class_info *ci = 
+        (struct rank_class_info *) xmalloc (sizeof(*ci));
 
-    yaz_log (LOG_DEBUG, "rank-1 create");
+    if (!log_initialized)
+    {
+        log_level = yaz_log_module_level("rank1");
+        log_initialized = 1;
+    }
+    yaz_log(log_level, "rank-1 create");
     return ci;
 }
 
@@ -83,54 +91,57 @@ static void destroy (struct zebra_register *reg, void *class_handle)
 {
     struct rank_class_info *ci = (struct rank_class_info *) class_handle;
 
-    yaz_log (LOG_DEBUG, "rank-1 destroy");
+    yaz_log(log_level, "rank-1 destroy");
     xfree (ci);
 }
 
 
-/*
+/**
  * begin: Prepares beginning of "real" ranking. Called once for
  *  each result set. The returned handle is a "set handle" and
  *  will be used in each of the handlers below.
  */
-static void *begin (struct zebra_register *reg, void *class_handle, RSET rset)
+static void *begin (struct zebra_register *reg, 
+                    void *class_handle, RSET rset, NMEM nmem,
+                    TERMID *terms, int numterms)
 {
-    struct rank_set_info *si = (struct rank_set_info *) xmalloc (sizeof(*si));
+    struct rank_set_info *si = 
+        (struct rank_set_info *) nmem_malloc (nmem,sizeof(*si));
     int i;
 
-#if DEBUG_RANK
-    yaz_log (LOG_LOG, "rank-1 begin");
-#endif
-    si->no_entries = rset->no_rset_terms;
+    yaz_log(log_level, "rank-1 begin");
+    si->no_entries = numterms;
     si->no_rank_entries = 0;
+    si->nmem=nmem;
     si->entries = (struct rank_term_info *)
-	xmalloc (sizeof(*si->entries)*si->no_entries);
-    for (i = 0; i < si->no_entries; i++)
+	nmem_malloc (si->nmem, sizeof(*si->entries)*numterms); 
+    for (i = 0; i < numterms; i++)
     {
-	int g = rset->rset_terms[i]->nn;
-#if DEBUG_RANK
-        yaz_log(LOG_LOG, "i=%d flags=%s", i, rset->rset_terms[i]->flags);
-#endif
-	if (!strncmp (rset->rset_terms[i]->flags, "rank,", 5))
+	zint g = rset_count(terms[i]->rset);
+        yaz_log(log_level, "i=%d flags=%s '%s'", i, 
+                terms[i]->flags, terms[i]->name );
+	if  (!strncmp (terms[i]->flags, "rank,", 5)) 
 	{
-            const char *cp = strstr(rset->rset_terms[i]->flags+4, ",w=");
+            const char *cp = strstr(terms[i]->flags+4, ",w=");
 	    si->entries[i].rank_flag = 1;
             if (cp)
                 si->entries[i].rank_weight = atoi (cp+3);
             else
-                si->entries[i].rank_weight = 34;
-#if DEBUG_RANK
-            yaz_log (LOG_LOG, " i=%d weight=%d", i,
-                     si->entries[i].rank_weight);
-#endif
+              si->entries[i].rank_weight = 34; /* sqrroot of 1000 */
+            yaz_log(log_level, " i=%d weight=%d g="ZINT_FORMAT, i,
+                     si->entries[i].rank_weight, g);
 	    (si->no_rank_entries)++;
 	}
 	else
 	    si->entries[i].rank_flag = 0;
-	si->entries[i].local_occur = 0;
+	si->entries[i].local_occur = 0;  /* FIXME */
 	si->entries[i].global_occur = g;
 	si->entries[i].global_inv = 32 - log2_int (g);
-	yaz_log (LOG_DEBUG, " global_inv = %d g = %d", 32 - log2_int (g), g);
+	yaz_log(log_level, " global_inv = %d g = " ZINT_FORMAT, 
+                (int) (32-log2_int (g)), g);
+        si->entries[i].term = terms[i];
+        si->entries[i].term_index=i;
+        terms[i]->rankpriv = &(si->entries[i]);
     }
     return si;
 }
@@ -141,25 +152,32 @@ static void *begin (struct zebra_register *reg, void *class_handle, RSET rset)
  */
 static void end (struct zebra_register *reg, void *set_handle)
 {
-    struct rank_set_info *si = (struct rank_set_info *) set_handle;
-    yaz_log (LOG_DEBUG, "rank-1 end");
-    xfree (si->entries);
-    xfree (si);
+    yaz_log(log_level, "rank-1 end");
+    /* no need to free anything, they are in nmems */
 }
 
-/*
+
+/**
  * add: Called for each word occurence in a result set. This routine
  *  should be as fast as possible. This routine should "incrementally"
  *  update the score.
  */
-static void add (void *set_handle, int seqno, int term_index)
+static void add (void *set_handle, int seqno, TERMID term)
 {
     struct rank_set_info *si = (struct rank_set_info *) set_handle;
-#if DEBUG_RANK
-    yaz_log (LOG_LOG, "rank-1 add seqno=%d term_index=%d", seqno, term_index);
-#endif
+    struct rank_term_info *ti;
+    assert(si);
+    if (!term)
+    {
+        yaz_log(log_level, "rank-1 add NULL term");
+        return;
+    }
+    ti= (struct rank_term_info *) term->rankpriv;
+    assert(ti);
     si->last_pos = seqno;
-    si->entries[term_index].local_occur++;
+    ti->local_occur++;
+    yaz_log(log_level, "rank-1 add seqno=%d term=%s count=%d", 
+            seqno, term->name,ti->local_occur);
 }
 
 /*
@@ -168,34 +186,29 @@ static void add (void *set_handle, int seqno, int term_index)
  *  score should be between 0 and 1000. If score cannot be obtained
  *  -1 should be returned.
  */
-static int calc (void *set_handle, int sysno)
+static int calc (void *set_handle, zint sysno, zint staticrank,
+		 int *stop_flag)
 {
     int i, lo, divisor, score = 0;
     struct rank_set_info *si = (struct rank_set_info *) set_handle;
 
-    if (!si->no_rank_entries)
-	return -1;
+    if (!si->no_rank_entries) 
+	return -1;   /* ranking not enabled for any terms */
 
-#if DEBUG_RANK
-    yaz_log(LOG_LOG, "calc");
-#endif
     for (i = 0; i < si->no_entries; i++)
     {
-#if DEBUG_RANK
-        yaz_log(LOG_LOG, "i=%d rank_flag=%d lo=%d",
+        yaz_log(log_level, "calc: i=%d rank_flag=%d lo=%d",
                 i, si->entries[i].rank_flag, si->entries[i].local_occur);
-#endif
 	if (si->entries[i].rank_flag && (lo = si->entries[i].local_occur))
 	    score += (8+log2_int (lo)) * si->entries[i].global_inv *
                 si->entries[i].rank_weight;
     }
     divisor = si->no_rank_entries * (8+log2_int (si->last_pos/si->no_entries));
     score = score / divisor;
-#if DEBUG_RANK
-    yaz_log (LOG_LOG, "sysno=%d score=%d", sysno, score);
-#endif
+    yaz_log(log_level, "calc sysno=" ZINT_FORMAT " score=%d", sysno, score);
     if (score > 1000)
 	score = 1000;
+    /* reset the counts for the next term */
     for (i = 0; i < si->no_entries; i++)
 	si->entries[i].local_occur = 0;
     return score;
@@ -229,4 +242,12 @@ static struct rank_control rank_control = {
     add,
 };
  
-struct rank_control *rank1_class = &rank_control;
+struct rank_control *rank_1_class = &rank_control;
+/*
+ * Local variables:
+ * c-basic-offset: 4
+ * indent-tabs-mode: nil
+ * End:
+ * vim: shiftwidth=4 tabstop=8 expandtab
+ */
+
